"""rename-processing-progress-model

Revision ID: 005
Revises: 004
Create Date: 2025-01-17 20:29:55.674674

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "005"
down_revision: Union[str, None] = "004"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.rename_table("processing_progress", "document_processing_progress")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.rename_table("document_processing_progress", "processing_progress")
    # ### end Alembic commands ###
