@import url("https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap");.TextEditor_tiptap__x4zbW{background:#0000;border-radius:0 0 5px 5px;margin-bottom:1rem;caret-color:#0070f3;font-family:Inter,sans-serif;position:relative;width:100%}.TextEditor_tiptap__x4zbW>*+*{margin-top:0}.TextEditor_tiptap__x4zbW ul{padding-left:1.5rem;list-style:disc}.TextEditor_tiptap__x4zbW ol{padding-left:1.5rem;list-style:decimal}.TextEditor_tiptap__x4zbW h1,.TextEditor_tiptap__x4zbW h2,.TextEditor_tiptap__x4zbW h3,.TextEditor_tiptap__x4zbW h4,.TextEditor_tiptap__x4zbW h5,.TextEditor_tiptap__x4zbW h6{font-weight:600;line-height:1.2;margin-bottom:1rem}.TextEditor_tiptap__x4zbW h1{font-size:2.5rem}.TextEditor_tiptap__x4zbW h2{font-size:2rem}.TextEditor_tiptap__x4zbW h3{font-size:1.75rem}.TextEditor_tiptap__x4zbW h4,.TextEditor_tiptap__x4zbW h5,.TextEditor_tiptap__x4zbW h6{font-size:1rem}.TextEditor_tiptap__x4zbW h4{margin-bottom:1.33rem}.TextEditor_tiptap__x4zbW h5{margin-bottom:1.67rem}.TextEditor_tiptap__x4zbW h6{margin-bottom:2.33rem}.TextEditor_tiptap__x4zbW code{background-color:#f5f5f5;border-radius:.25rem;color:#0070f3;font-size:.875rem;padding:.2rem .4rem}.TextEditor_tiptap__x4zbW pre{background:#f5f5f5;color:#333;font-family:JetBrainsMono,monospace;padding:1rem;border-radius:.5rem;width:100%}.TextEditor_tiptap__x4zbW code{color:inherit;padding:0;background:none;font-size:.8rem}.TextEditor_tiptap__x4zbW blockquote{padding-left:1rem;border-left:4px solid #0070f3;color:#333;font-style:italic}.TextEditor_tiptap__x4zbW hr{border:none;border-top:1px solid #eaeaea;margin:2rem 0;width:100%}.TextEditor_fullScreenEditor__Afw4n{position:fixed;top:0;left:0;width:100vw;background-color:#fff;z-index:1000;padding:20px;box-sizing:border-box;overflow-y:auto;min-height:60vh}.TextEditor_tiptap__x4zbW .ProseMirror{outline:none;width:100%}.TextEditor_tiptap__x4zbW .ProseMirror p{margin:0}.TextEditor_tiptap__x4zbW .ProseMirror>*+*{margin-top:.75em}.TextEditor_truncatedEditor__SP6RG{max-height:150px;overflow-y:auto;text-overflow:ellipsis;white-space:normal}.TextEditor_fullScreenEditor__Afw4n{height:100vh;overflow:auto}